# -*- coding: utf-8 -*-
import smbus
import sys
from time import sleep
import requests
from datetime import datetime

# Define some device parameters
I2C_ADDR  = 0x27 # I2C device address, if any error, change this address to 0x27
LCD_WIDTH = 20 # Maximum characters per line

# Define some device constants
LCD_CHR = 1 # Mode - Sending data
LCD_CMD = 0 # Mode - Sending command

LCD_LINE_1 = 0x80 # LCD RAM address for the 1st line
LCD_LINE_2 = 0xC0 # LCD RAM address for the 2nd line
LCD_LINE_3 = 0x94 # LCD RAM address for the 3rd line
LCD_LINE_4 = 0xD4 # LCD RAM address for the 4th line

LCD_BACKLIGHT  = 0x08  # On
#LCD_BACKLIGHT = 0x00  # Off

ENABLE = 0b00000100 # Enable bit

# Timing constants
E_PULSE = 0.0005
E_DELAY = 0.0005

#Open I2C interface
#bus = smbus.SMBus(0)  # Rev 1 Pi uses 0
bus = smbus.SMBus(1) # Rev 2 Pi uses 1


# BME280-----------------------------------------------------
BME_address = 0x76
 
digT = []
digP = []
digH = []
 
t_fine = 0.0
 
 
def writeReg(reg_address, data):
    bus.write_byte_data(BME_address,reg_address,data)
 
def get_calib_param():
    calib = []
     
    for i in range (0x88,0x88+24):
        calib.append(bus.read_byte_data(BME_address,i))
    calib.append(bus.read_byte_data(BME_address,0xA1))
    for i in range (0xE1,0xE1+7):
        calib.append(bus.read_byte_data(BME_address,i))
 
    digT.append((calib[1] << 8) | calib[0])
    digT.append((calib[3] << 8) | calib[2])
    digT.append((calib[5] << 8) | calib[4])
    digP.append((calib[7] << 8) | calib[6])
    digP.append((calib[9] << 8) | calib[8])
    digP.append((calib[11]<< 8) | calib[10])
    digP.append((calib[13]<< 8) | calib[12])
    digP.append((calib[15]<< 8) | calib[14])
    digP.append((calib[17]<< 8) | calib[16])
    digP.append((calib[19]<< 8) | calib[18])
    digP.append((calib[21]<< 8) | calib[20])
    digP.append((calib[23]<< 8) | calib[22])
    digH.append( calib[24] )
    digH.append((calib[26]<< 8) | calib[25])
    digH.append( calib[27] )
    digH.append((calib[28]<< 4) | (0x0F & calib[29]))
    digH.append((calib[30]<< 4) | ((calib[29] >> 4) & 0x0F))
    digH.append( calib[31] )
     
    for i in range(1,2):
        if digT[i] & 0x8000:
            digT[i] = (-digT[i] ^ 0xFFFF) + 1
 
    for i in range(1,8):
        if digP[i] & 0x8000:
            digP[i] = (-digP[i] ^ 0xFFFF) + 1
 
    for i in range(0,6):
        if digH[i] & 0x8000:
            digH[i] = (-digH[i] ^ 0xFFFF) + 1 
 
def BME_readData():
    data = []
    for i in range (0xF7, 0xF7+8):
        data.append(bus.read_byte_data(BME_address,i))
    pres_raw = (data[0] << 12) | (data[1] << 4) | (data[2] >> 4)
    temp_raw = (data[3] << 12) | (data[4] << 4) | (data[5] >> 4)
    hum_raw  = (data[6] << 8)  |  data[7]
     
    #compensate_T(temp_raw)
    #compensate_P(pres_raw)
    #compensate_H(hum_raw)
    t = compensate_T(temp_raw)
    p = compensate_P(pres_raw)
    h = compensate_H(hum_raw)
    return p + 'hPa,' + t+degree + ',' + h+'%'
 
def compensate_P(adc_P):
    global  t_fine
    pressure = 0.0
     
    v1 = (t_fine / 2.0) - 64000.0
    v2 = (((v1 / 4.0) * (v1 / 4.0)) / 2048) * digP[5]
    v2 = v2 + ((v1 * digP[4]) * 2.0)
    v2 = (v2 / 4.0) + (digP[3] * 65536.0)
    v1 = (((digP[2] * (((v1 / 4.0) * (v1 / 4.0)) / 8192)) / 8)  + ((digP[1] * v1) / 2.0)) / 262144
    v1 = ((32768 + v1) * digP[0]) / 32768
     
    if v1 == 0:
        return 0
    pressure = ((1048576 - adc_P) - (v2 / 4096)) * 3125
    if pressure < 0x80000000:
        pressure = (pressure * 2.0) / v1
    else:
        pressure = (pressure / v1) * 2
    v1 = (digP[8] * (((pressure / 8.0) * (pressure / 8.0)) / 8192.0)) / 4096
    v2 = ((pressure / 4.0) * digP[7]) / 8192.0
    pressure = pressure + ((v1 + v2 + digP[6]) / 16.0)  
 
    # print 'pressure : %7.2f hPa' % (pressure/100)
    return '%5.0f' % (pressure/100)
 
 
def compensate_T(adc_T):
    global t_fine
    v1 = (adc_T / 16384.0 - digT[0] / 1024.0) * digT[1]
    v2 = (adc_T / 131072.0 - digT[0] / 8192.0) * (adc_T / 131072.0 - digT[0] / 8192.0) * digT[2]
    t_fine = v1 + v2
    temperature = t_fine / 5120.0
    # print 'temp : %-6.2f ℃' % (temperature) 
    return '%.1f' % (temperature) 
 
def compensate_H(adc_H):
    global t_fine
    var_h = t_fine - 76800.0
    if var_h != 0:
        var_h = (adc_H - (digH[3] * 64.0 + digH[4]/16384.0 * var_h)) * (digH[1] / 65536.0 * (1.0 + digH[5] / 67108864.0 * var_h * (1.0 + digH[2] / 67108864.0 * var_h)))
    else:
        return 0
    var_h = var_h * (1.0 - digH[0] * var_h / 524288.0)
    if var_h > 100.0:
        var_h = 100.0
    elif var_h < 0.0:
        var_h = 0.0
    # print 'hum : %6.2f ％' % (var_h)
    return '%.1f' % (var_h)
 
def BME_setup():
    osrs_t = 1            #Temperature oversampling x 1
    osrs_p = 1            #Pressure oversampling x 1
    osrs_h = 1            #Humidity oversampling x 1
    mode   = 3            #Normal mode
    t_sb   = 5            #Tstandby 1000ms
    filter = 0            #Filter off
    spi3w_en = 0            #3-wire SPI Disable
 
    ctrl_meas_reg = (osrs_t << 5) | (osrs_p << 2) | mode
    config_reg    = (t_sb << 5) | (filter << 2) | spi3w_en
    ctrl_hum_reg  = osrs_h
 
    writeReg(0xF2,ctrl_hum_reg)
    writeReg(0xF4,ctrl_meas_reg)
    writeReg(0xF5,config_reg)

# LCD ---------------------------------------------------------
def lcd_init():
  # Initialise display
  lcd_byte(0x33,LCD_CMD) # 110011 Initialise
  lcd_byte(0x32,LCD_CMD) # 110010 Initialise
  lcd_byte(0x06,LCD_CMD) # 000110 Cursor move direction
  lcd_byte(0x0C,LCD_CMD) # 001100 Display On,Cursor Off, Blink Off 
  lcd_byte(0x28,LCD_CMD) # 101000 Data length, number of lines, font size
  lcd_byte(0x01,LCD_CMD) # 000001 Clear display
  sleep(E_DELAY)

def lcd_byte(bits, mode):
  # Send byte to data pins
  # bits = the data
  # mode = 1 for data
  #        0 for command

  bits_high = mode | (bits & 0xF0) | LCD_BACKLIGHT
  bits_low = mode | ((bits<<4) & 0xF0) | LCD_BACKLIGHT

  # High bits
  bus.write_byte(I2C_ADDR, bits_high)
  lcd_toggle_enable(bits_high)

  # Low bits
  bus.write_byte(I2C_ADDR, bits_low)
  lcd_toggle_enable(bits_low)

def lcd_toggle_enable(bits):
  # Toggle enable
  sleep(E_DELAY)
  bus.write_byte(I2C_ADDR, (bits | ENABLE))
  sleep(E_PULSE)
  bus.write_byte(I2C_ADDR,(bits & ~ENABLE))
  sleep(E_DELAY)

def lcd_string(message,line):
  # Send string to display
  message = message.ljust(LCD_WIDTH," ")
  lcd_byte(line, LCD_CMD)
  for i in range(LCD_WIDTH):
    lcd_byte(ord(message[i]),LCD_CHR)

def check_writable(c):
    if c >= 0x06 and c <= 0xff :
        return c
    else:
        return 0x20 # 空白文字

try:
    unicode # python2
    def u(str): return str.decode('utf-8')
    pass
except: # python3
    def u(str): return str
    pass

# 最低気温と最高気温を「/」区切りの文字列に
def getLCDMinMax(temp):
    if temp['min'] != None:
        min = '{0}'.format(temp['min']['celsius'])+degree
    else:
        min = 'MIN'

    if temp['max'] != None:
        max = '{0}'.format(temp['max']['celsius'])+degree
    else:
        max = 'MAX'
    return min + '/' + max

# 天気をカタカナの文字列に
def getLCDWeather(weather):
    if weather.find(u('のち')) != -1:
        datas = weather.split(u('のち')) 
        return replaceWeather(datas[0])+'>'+replaceWeather(datas[1]) # →
    elif weather.find(u('時々')) != -1:
        datas = weather.split(u('時々'))
        return replaceWeather(datas[0])+'/'+replaceWeather(datas[1])
    else:
        return replaceWeather(weather)

# 各天気をカタカナに
def replaceWeather(weather):
    if weather.find(u('晴')) != -1:
        return ha+re # ハレ
    elif weather.find(u('曇')) != -1:
        return ku+mo+ri # クモリ
    elif weather.find(u('雨')) != -1:
        return a1+me # アメ
    elif weather.find(u('雪')) != -1:
        return yu+ki # ユキ
    else:
        return '---'

# Webサービスより天気予報を取得
def getWeather():
    global minmax, weather, minmax2, weather2
    location = 260010  # 京都
    proxies = None
    # proxy環境下の方は下記を記述して有効に
    #proxies = {
    #    'http': 'プロキシサーバ名:ポート番号',
    #    'https': 'プロキシサーバ名:ポート番号'
    #}
    try:
        weather_json = requests.get(
            'http://weather.livedoor.com/forecast/webservice/json/v1?city={0}'.format(location), proxies=proxies, timeout=5
        ).json()
        print('インターネットから天気予報データを入手しました')
    except requests.exceptions.RequestException:
        print('ネットワークエラー')
        if second==0:
            sys.exit()
    for forecast in weather_json['forecasts']:
        if forecast['dateLabel'] == u('今日'):
            minmax = getLCDMinMax(forecast['temperature'])
            weather = getLCDWeather(forecast['telop'])
        if forecast['dateLabel'] == u('明日'):
            minmax2 = getLCDMinMax(forecast['temperature'])
            weather2 = getLCDWeather(forecast['telop'])

# 天気情報をLCDに表示
def displayWeather():
    # 現在時刻
    title1 = ki+yo2+u1+to+datetime.now().strftime('%Y-%m%d-%H%M%S')
    lcd_string(title1,LCD_LINE_1)
  
    lcd_string(BME_Data,LCD_LINE_2)
    s=''
    day = ki+yo2+u1+' ' # キョウ
    #for i in range(chars_per_line-len(day)-len(s)):
    #    s = ' '+s
    lcd_string(day+weather+' '+minmax,LCD_LINE_3)

    s=''
    day = a1+su+'  ' # アス 
    #for i in range(chars_per_line-len(day)-len(s)):
    #    s = ' '+s
    lcd_string(day+s+weather2+' '+minmax2,LCD_LINE_4)

a1=chr(0xb1)
i1=chr(0xb2)
u1=chr(0xb3)
e1=chr(0xb4)
o1=chr(0xb5)

ka=chr(0xb6)
ki=chr(0xb7)
ku=chr(0xb8)
ke=chr(0xb9)
ko=chr(0xba)

sa=chr(0xbb)
si=chr(0xbc)
su=chr(0xbd)
se=chr(0xbe)
so=chr(0xbf)

ta=chr(0xc0)
ti=chr(0xc1)
tu=chr(0xc2)
te=chr(0xc3)
to=chr(0xc4)

na=chr(0xc5)
ni=chr(0xc6)
nu=chr(0xc7)
ne=chr(0xc8)
no=chr(0xc9)

ha=chr(0xca)
hi=chr(0xcb)
hu=chr(0xcc)
he=chr(0xcd)
ho=chr(0xce)

ma=chr(0xcf)
mi=chr(0xd0)
mu=chr(0xd1)
me=chr(0xd2)
mo=chr(0xd3)

ya=chr(0xd4)
yu=chr(0xd5)
yo=chr(0xd6)

ra=chr(0xd7)
ri=chr(0xd8)
ru=chr(0xd9)
re=chr(0xda)
ro=chr(0xdb)

wa=chr(0xdc)
wo=chr(0xa6)

kagi1=chr(0xa2)
kagi2=chr(0xa3)


n1=chr(0xdd)


maru=chr(0xa1)
ten=chr(0xa4)

ten2=chr(0xde)
maru2=chr(0xdf)

dot=chr(0xa5)

a2=chr(0xa7)
i2=chr(0xa8)
u2=chr(0xa9)
e2=chr(0xaa)
o2=chr(0xab)

ya2=chr(0xac)
yu2=chr(0xad)
yo2=chr(0xae)
tu2=chr(0xaf)

degree=chr(0xdf)

chars_per_line = 20
display_lines = 4
display_chars = chars_per_line*display_lines


# Initialise display
lcd_init()
temperature = 999
minmax = ''
weather = ''
minmax2 = ''
weather2 = ''
BME_Data=''
BME_setup()
get_calib_param()

second = 0  # プログラム起動時から経過した秒数(目安)
hour = 0    # プログラム起動時から経過した時間(目安)
prevhour = -1  # 前回天気予報を問い合わせた時間

try:
    while True:
    
        if hour != prevhour: # 前回問い合わせより約1時間後
            getWeather()     # 天気予報取得
            prevhour = hour

        try:
            BME_Data = BME_readData() # 温度取得
        except IOError:
            BME_Data='Weather Data failure'  # 温度取得失敗

        try:
            displayWeather()  # LCDに情報表示
        except IOError:
            print('I2C通信エラースキップ')

        second = second + 1 # 1秒進める 
        hour = int(second/3600)  # 秒を時間に変換(0,1,2,…のように整数のみ)

        sleep(1)

except KeyboardInterrupt:
    pass

#GPIO.cleanup()
